import boto3
import requests
import json
from psc_orgkey import decode_orgkey
import logging
import os

# TODO DSER-31275 - Once the RBAC migration jobs have been fully migrated to gitlab rather than jenkins then we will be able to hit artifactory from the gitlab job.
#  Therefore we can pull the orgkey library from artifactory and then delete the psc_orgKey library that we had to drag under db/secops.
#  To switch to the artifactory version the only change should be adding psc-orgkey to the requirements and tweaking the way we install requirements
#  in the prod_bootstrap_and_migrate_rbac script to use a proxy or trusted url

logger = logging.getLogger("add_subroles_to_custom_roles")
logger.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s %(levelname)s - %(message)s")
handler_console = logging.StreamHandler()
handler_console.setFormatter(formatter)
logger.addHandler(handler_console)


def main(plan, env, x_auth_token):

    # Get the roles table and region for the environment
    roles_table, region = get_table(env)

    # Get env vars for the subroles to add and the specific subrole to find custom roles for
    subrole_to_find = os.environ.get('SUBROLE_TO_FIND')
    subroles_to_add = os.environ.get('SUBROLES_TO_ADD')

    if not is_valid_subrole_urn(subrole_to_find) or not is_valid_subrole_urn(subroles_to_add):
        raise Exception("Subrole to find {} is invalid or subrole to add {} is invalid".format(subrole_to_find, subroles_to_add))

    find_roles_with_specific_subrole(env, region, roles_table, subrole_to_find, subroles_to_add, plan, x_auth_token)


def find_roles_with_specific_subrole(env, region, roles_table, subrole, subrole_additions, dry_mode, x_auth_token):

    # If hitting local db
    if roles_table == "localdev_access_roles":
        dynamodb = boto3.resource('dynamodb',
                                  region_name="us-east-1",
                                  endpoint_url='https://access-mockaws:4569',
                                  verify=False)
    else:
        dynamodb = boto3.resource('dynamodb', region_name=region)

    table = dynamodb.Table(roles_table)
    response = table.scan()
    data = response['Items']

    while 'LastEvaluatedKey' in response:
        response = table.scan(ExclusiveStartKey=response['LastEvaluatedKey'])
        data.extend(response['Items'])

    roles_with_specific_subrole_in_hub = []

    for item in data:
        role_urn = item.get("urn")
        child__urn = item.get("child_urn")

        if child__urn == subrole:
            if is_custom_role_urn(role_urn):
                roles_with_specific_subrole_in_hub.append(role_urn)


    filtered_roles_for_environment = []
    # Decode each org key and determine if relevant to environment
    for role in roles_with_specific_subrole_in_hub:
        org_key = get_org_key_from_role_urn(role)
        org_key_env = get_env_from_org_key(org_key)
        # Extra check needed for local
        if org_key_env == env or (org_key_env == "local" and env == "localdev"):
            filtered_roles_for_environment.append(role)

    logger.info("Number of custom roles with specific subrole in hub: %s", len(roles_with_specific_subrole_in_hub))
    logger.info("Number of custom roles with specific subrole in {}: %s".format(env), len(filtered_roles_for_environment))
    logger.debug("Custom roles with specific subrole: %s", filtered_roles_for_environment)
    logger.info("===================================")

    add_subroles_to_role(env, filtered_roles_for_environment, subrole_additions, dry_mode, x_auth_token)


def get_number_of_custom_roles_with_specific_subrole_per_org_key(roles_with_specific_subrole_in_hub):
    # Helpful function to find number of custom roles with specific subrole per orgkey.
    org_key_to_role_dict = {}
    for custom_role in roles_with_specific_subrole_in_hub:
        custom_role_org_key = get_org_key_from_role_urn(custom_role)
        if custom_role_org_key in org_key_to_role_dict:
            org_key_to_role_dict[custom_role_org_key].append(custom_role)
        else:
            org_key_to_role_dict[custom_role_org_key] = [custom_role]

    logger.info(org_key_to_role_dict)

    my_list = []
    count = 0
    for key in org_key_to_role_dict:
        my_list.append((key, len(org_key_to_role_dict[key])))
        count += len(org_key_to_role_dict[key])
    logger.info(my_list)
    logger.info(count)


def get_org_key_from_role_urn(role_urn):
    return role_urn.split(":")[2]


def verify_addition_needed(json_resp, role, subrole_additions):
    subrole_additions_list = subrole_additions.split(",")
    verified_subroles_to_add = []
    data = json_resp.get('subroles')
    for subrole in subrole_additions_list:
        # Checking if subrole already exists for this role
        if not any(dict.get('child_urn') == subrole for dict in data):
            logger.debug("Need to add {} to {}.".format(subrole, role))
            verified_subroles_to_add.append(subrole)
        else:
            logger.debug("{} already exists on {}. No addition needed.".format(subrole, role))

    return verified_subroles_to_add


def format_subroles_to_add(role_urn, verified_subroles_to_add):
    formatted_subroles_as_dicts = []
    for subrole in verified_subroles_to_add:
        subrole_dict = {
            "urn": role_urn,
            "child_urn": subrole
        }
        formatted_subroles_as_dicts.append(subrole_dict)
    return formatted_subroles_as_dicts


def get_request(path, get_headers):
    try:
        response = requests.get(path, headers=get_headers)
        if 400 <= response.status_code < 600:
            logger.info('Call to get subroles for role failed. Response code = {}. Body = {}'.format(response.status_code, str(response.content)))
        return response

    except Exception as e:
        logger.error('Error occurred while making call to get subroles for role.')
        logger.exception(e)


def post_request(path, request_body, post_headers):
    try:
        response = requests.post(path, data=request_body, headers=post_headers)
        if 400 <= response.status_code < 600:
            logger.info('Call to add subroles to role failed. Response code = {}. Body = {}'.format(response.status_code, str(response.content)))
        logger.info('Successfully added subroles!')
        return response

    except Exception as e:
        logger.error('Error occurred while making call to post new subroles for role.')
        logger.exception(e)


def add_subroles_to_role(env, roles_to_update, subrole_additions, dry_mode, x_auth_token):
    counter = 0
    # For each role that contains a specific subrole, Call API to get current subroles for custom role,
    # verify it needs the subroles to add, then do the addition and verify all roles were updated as expected
    orphaned_roles = []
    env_url = get_env_url_from_env(env)

    if env == "localdev":
        prefix = "http"
    else:
        prefix = "https"

    for role_urn in roles_to_update:
        org_key = get_org_key_from_role_urn(role_urn)
        path = "{}://{}/access/v2/orgs/{}/roles/{}/subroles/list".format(prefix, env_url, org_key, role_urn)
        get_headers = {'x-auth-token': x_auth_token}
        post_headers = {'x-auth-token': x_auth_token, 'content-type': "application/json;charset=UTF-8"}

        # Make get request
        resp = get_request(path, get_headers)

        # If the response status code is not a 200 just skip it and add the role to a list
        if resp.status_code != 200:
            orphaned_roles.append(role_urn)
            continue
        current_subroles_resp = resp.json()

        logger.info("Proceeding to verify if {} needs additional subroles.".format(role_urn))

        # Verify relevant subroles need to be added to the role
        verified_subroles_to_add = verify_addition_needed(current_subroles_resp, role_urn, subrole_additions)

        # If subroles need to be added to role call the Post API
        if len(verified_subroles_to_add) > 0:
            # create necessary request body w/ existing subroles that are on role + new subroles to be added
            formatted_subrole_additions = format_subroles_to_add(role_urn, verified_subroles_to_add)
            current_subroles = current_subroles_resp.get('subroles')
            subroles_request_body = current_subroles + formatted_subrole_additions

            # Call API to add new subroles to the role
            subroles_request_body_dict = {"subroles": subroles_request_body}
            subroles_request_body_json = json.dumps(subroles_request_body_dict)
            if dry_mode:
                logger.info("Dry Run: Would have added {} to {}".format(verified_subroles_to_add, role_urn))
                counter += 1
            else:
                post_resp = post_request(path, subroles_request_body_json, post_headers)
                post_json_resp = post_resp.json()
                logger.debug("After addition, subroles on role: {}".format(post_json_resp.get('subroles')))
                logger.info("Added {} to {}".format(verified_subroles_to_add, role_urn))
                counter += 1

    if dry_mode:
        logger.info("DRY RUN ONLY - Total roles that would have updated: {}.".format(counter))
        logger.info("Orphaned Roles: %s", orphaned_roles)
    else:
        logger.info("Total roles updated: %s", counter)
        logger.info("Orphaned Roles or custom roles that were not updated during migration: %s", orphaned_roles)


def is_custom_role_urn(role):
    roleParts = role.split(":")
    return len(roleParts) == 4 and role.startswith('psc:role') and len(roleParts[2]) > 0 and roleParts[3] is not None


def is_valid_subrole_urn(subrole_urn):
    if subrole_urn is None or subrole_urn == '':
        logger.error("Subrole urn cannot be empty")
        return False
    subroles_to_list = subrole_urn.split(",")
    for subrole in subroles_to_list:
        subroleParts = subrole.split(":")
        if not(len(subroleParts) == 3 and subrole.startswith('psc:subrole') and subroleParts[2] is not None):
            logger.error("{} not in the correct format".format(subrole))
            return False
    return True


def get_env_from_org_key(org_key):
    decoded = decode_orgkey(str(org_key))
    return decoded.env


def get_env_url_from_env(env):
    env_url_mapper = {
        "localdev": "localhost:8900",
        "dev01": "defense-dev01.cbdtest.io",
        "test03": "defense-test03.cbdtest.io",
        "eap01": "defense-eap01.conferdeploy.net",
        "prod01": "dashboard.confer.net",
        "prod02": "defense.conferdeploy.net",
        "prod05": "defense-prod05.conferdeploy.net",
        "prod06": "defense-eu.conferdeploy.net",
        "prodnrt": "defense-prodnrt.conferdeploy.net",
        "prodsyd": "defense-prodsyd.conferdeploy.net"
    }
    return env_url_mapper.get(env)


def get_table(env):
    hub = None
    region = None
    localdev_hub = ['localdev']
    dev_hub = ['dev01']
    test_hub = ['test03']
    prod_us_hub = ['eap01', 'prod01', 'prod02', 'prod05']
    prod_eu_hub = ['prod06']
    prod_ap_hub = ['prodnrt']
    prod_ap2_hub = ['prodsyd']

    if env in localdev_hub:
        hub = 'localdev'
        region = "us-east-1"
    elif env in dev_hub:
        hub = 'hub-dev-us'
        region = "us-east-1"
    elif env in test_hub:
        hub = 'hub04'
        region = "us-east-1"
    elif env in prod_us_hub:
        hub = 'hub-prod-us'
        region = "us-east-1"
    elif env in prod_eu_hub:
        hub = 'hub-prod-eu'
        region = "eu-central-1"
    elif env in prod_ap_hub:
        hub = 'hub-prod-ap'
        region = "ap-northeast-1"
    elif env in prod_ap2_hub:
        hub = 'hub-prod-ap2'
        region = "ap-southeast-2"

    if hub == None:
        raise Exception("Could not get Hub configuration for environment {}".format(env))
    logger.info("Hub Environment %s " % hub)

    return "{}_access_roles".format(hub), region

if __name__ == '__main__':
    main()