import boto3
import os
import csv

env_to_hub_map = {
    "dev01": "hub-dev-us",
    "test03": "hub04",
    "eap01": "hub-prod-us",
    "prod01": "hub-prod-us",
    "prod02": "hub-prod-us",
    "prod05": "hub-prod-us",
    "prod06": "hub-prod-eu",
    "prodnrt": "hub-prod-ap",
    "prodsyd": "hub-prod-ap2"
}

hub_to_region_map = {
    "hub-dev-us": "us-east-1",
    "hub04": "us-east-1",
    "hub-prod-us": "us-east-1",
    "hub-prod-eu": "eu-central-1",
    "hub-prod-ap": "ap-northeast-1",
    "hub-prod-ap2": "ap-southeast-2"
}

path = ""

# How to use
# 1. Go to Octa -> AWS -> S3 and search for the current csv with deregistered org data
# 2. Download and put those files in the input folder
# 3. To obtain your AWS credentials:
#    in .saml2aws file change
#       for prod (scargo)     role_arn = arn:aws:iam::132308400445:role/cb-team-csp-identity
#       for dev (confer-test) role_arn = arn:aws:iam::605728677638:role/cb-admin
#    run 'saml2aws login' and 'saml2aws script'
# 4. Scroll down to initialize_dynamo function and fill in your AWS credentials
# 5. At the bottom of the script are all the method invocations for read and write for each env. Uncomment the one you need.
# 6. To run the script you can use 'python3 integration/secops/deregistered_orgs_script/delete_grants_for_deregistered_org.py' ('brew install python3' if you don't have it)

def get_org_keys_from_csv(env):
    path_to_input_files = path + 'input/'
    all_input_files = os.listdir(path_to_input_files)

    file_names = [file for file in all_input_files if env in file and file.endswith(".csv")]

    if not file_names:
        print(f"No file for {env} in input list")
        return

    org_key_list = []
    for file_name in file_names:
        with open(path_to_input_files + file_name, "r") as input_file:
            file = csv.DictReader(input_file)

            for row in file:
                org_key = row['org_key']
                if org_key and (org_key not in org_key_list):
                    print("Org key: " + org_key)
                    org_key_list.append(org_key)

    return org_key_list


def read_grants_of_deregistered_orgs(env: str = "dev01"):
    if env not in env_to_hub_map:
        print("Wrong env value provided")
        return

    hub = env_to_hub_map[env]
    # Get org_keys from input CSV file
    org_key_list = get_org_keys_from_csv(env)

    # Get Dynamo Client using AWS credentials for specific env
    dynamodb = initialize_dynamo(hub_to_region_map[hub], True if env in ["dev01", "test03"] else False)
    grants_table = dynamodb.Table('{0}_access_grants'.format(hub))

    # Create a CSV file for the env and write header row
    with open(path + f'output/grants_{hub}_{env}.csv', "w+") as output_csv_file:
        csv_writer = csv.writer(output_csv_file, delimiter=',')
        csv_writer.writerow(['principal', 'org_ref', 'created_at', 'created_by', 'principal_name', 'revoked',
                             'roles', 'updated_at', 'updated_by', 'version', 'profiles'])

        # Iterate over org_key list
        for org_key in org_key_list:
            org_urn = f'psc:org:{org_key}'

            # Query Table data for particular org
            response = grants_table.query(
                IndexName="org-ref-index",
                KeyConditionExpression="org_ref = :org_urn",
                ExpressionAttributeValues={
                    ":org_urn": org_urn
                }
            )

            response_data = response['Items']
            print(f"Response of grants_table.query for org[{org_urn}] : ", *response_data, sep='\n')

            while 'LastEvaluatedKey' in response:
                response = grants_table.query(ExclusiveStartKey=response['LastEvaluatedKey'])
                response_data.extend(response['Items'])
                print("Response received")

            # Write the response data into CSV
            for item in response_data:
                csv_writer.writerow([item['principal'] if 'principal' in item else "",
                                     item['org_ref'] if 'org_ref' in item else "",
                                     item['created_at'] if 'created_at' in item else "",
                                     item['created_by'] if 'created_by' in item else "",
                                     item['principal_name'] if 'principal_name' in item else "",
                                     item['revoked'] if 'revoked' in item else "",
                                     item['roles'] if 'roles' in item else "",
                                     item['updated_at'] if 'updated_at' in item else "",
                                     item['updated_by'] if 'updated_by' in item else "",
                                     item['version'] if 'version' in item else "",
                                     item['profiles'] if 'profiles' in item else ""])


def delete_grant_by_reading_csv(env, delete_flag: bool = False):
    if env not in env_to_hub_map:
        print("Wrong env value provided")
        return

    hub = env_to_hub_map[env]
    # Get Dynamo Client using AWS credentials for specific env
    dynamodb = initialize_dynamo(hub_to_region_map[hub], True if env in ["dev01", "test03"] else False)
    grants_table = dynamodb.Table('{0}_access_grants'.format(hub))

    # Delete Grants data
    if delete_flag:
        with open(path + f'output/grants_{hub}_{env}.csv', "r") as input_csv_file:
            csv_reader = csv.reader(input_csv_file, delimiter=",")
            next(csv_reader)  # Skip the Header Row

            print(" ============================== Deletion in Progress ========================================== ")
            with grants_table.batch_writer() as batch:
                for item in csv_reader:
                    print(f" -- Deleting Grant for {item[0]} in org {item[1]} -- ")
                    batch.delete_item(
                        Key={
                            'principal': item[0],
                            'org_ref': item[1]
                        }
                    )
            print((csv_reader.line_num - 1), f' Grants deleted')
            print(" ============================== Deletion Completed ========================================== ")


# Obtain the required tokens via 'saml2aws script'
def initialize_dynamo(region, is_dev):
    if is_dev:
        dynamodb = boto3.resource('dynamodb',
                                  aws_session_token="",
                                  aws_access_key_id="",
                                  aws_secret_access_key="",
                                  region_name=region)
    else:
        dynamodb = boto3.resource('dynamodb',
                                  aws_session_token="",
                                  aws_access_key_id="",
                                  aws_secret_access_key="",
                                  region_name=region)
    return dynamodb


if __name__ == '__main__':
    # *** Read and store Grants in CSV ***
    read_grants_of_deregistered_orgs("dev01")
    # read_grants_of_deregistered_orgs("eap01")
    # read_grants_of_deregistered_orgs("prod01")
    # read_grants_of_deregistered_orgs("prod02")
    # read_grants_of_deregistered_orgs("prod05")
    # read_grants_of_deregistered_orgs("prod06")
    # read_grants_of_deregistered_orgs("prodnrt")
    # read_grants_of_deregistered_orgs("prodsyd")

    # *** Delete Grants from reading the csv files ***
    # delete_grant_by_reading_csv("dev01", True)
    # delete_grant_by_reading_csv("eap01", True)
    # delete_grant_by_reading_csv("prod01", True)
    # delete_grant_by_reading_csv("prod02", True)
    # delete_grant_by_reading_csv("prod05", True)
    # delete_grant_by_reading_csv("prod06", True)
    # delete_grant_by_reading_csv("prodnrt", True)
    # delete_grant_by_reading_csv("prodsyd", True)