import requests
import json
import urllib3
import logging
import argparse
import yaml
import os
import clients as app_clients

from botocore.exceptions import ClientError

urllib3.disable_warnings()
logger = logging.getLogger("migrate_secops_role_manager")
logger.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s %(levelname)s - %(message)s")
handler_console = logging.StreamHandler()
handler_console.setFormatter(formatter)
logger.addHandler(handler_console)

secops_role_manager_name = 'SECOPS_ROLE_MANAGER'

def main(environment=None, green_only=False):
    CURRENT_FILE_PATH = os.path.dirname(os.path.abspath(__file__))
    environment_choices = ['localdev', 'dev08', 'dev01', 'test03', 'eap01', 'prod01', 'prod02', 'prod04', 'prod05', 'prod06', 'prodnrt', 'prodsyd']
    if environment is not None:
        if environment not in environment_choices:
            raise ValueError('Incorrect environment choice')
    parser = argparse.ArgumentParser(description="Migrate Secops Role Manager")
    parser.add_argument('-e', '--environment', required=False,
                        choices=environment_choices)
    parser.add_argument("-g", "--green-only", help="Run migration hitting only green service", action="store_true")

    args = parser.parse_known_args()[0]

    env = args.environment if environment is None else environment
    if env not in environment_choices:
        raise ValueError('Incorrect environment choice')
    env_urls = yaml.load(open("{}/url_configs.yml".format(CURRENT_FILE_PATH)), Loader=yaml.BaseLoader)[env]
    es_url = env_urls['elasticsearch']
    dashboard_url = env_urls['dashboard']

    dynamo_client = app_clients.DynamoDbClient(env)
    dynamo_client.setClientEnvironment()
    es_client = app_clients.EsClient(es_url)
    es_client.setData()

    with open('../local/localdev-batchWriteSecops.json') as f:
        role_manager_definition = json.load(f)

    hub = get_hub(env)
    role_manager_definition['{}_access_roles'.format(hub)] = role_manager_definition.pop('localdev_access_roles')
    connector_dict = {'X-Auth-Token' : 'token', 'org-key' : 'org-key'}

    # Create Secops Role Manager definition and permissions in access roles table
    try:
        dynamo_client.batch_write(role_manager_definition)
    except ClientError as ex:
        logger.error("Batch Write failed for role manager, error: %s)" % ex.response)
    try:
        # Check if custom connector already exists
        manager_connector_exists = False
        query_response = es_client.queryConnector(secops_role_manager_name.lower()).json()
        if query_response['hits']['hits'] != []:
            if query_response['hits']['hits'][0]['_source']['name'] == secops_role_manager_name:
                logger.info("Secops Role Manager already exists, verifying grant is assigned")
                api_key = query_response['hits']['hits'][0]['_source']['apiKey']
                connector_id = query_response['hits']['hits'][0]['_source']['connectorId']
                logger.info("X-Auth-Token: %s/%s for role manager connector" % (api_key, connector_id))
                connector_dict['X-Auth-Token'] = "%s/%s" % (api_key, connector_id)
                connector_dict['org-key'] = get_org_key(dashboard_url, connector_dict['X-Auth-Token'])
                verify_correct_grant(dynamo_client, connector_dict['org-key'], connector_id, hub)
                manager_connector_exists = True
                return connector_dict
        elif not manager_connector_exists:
            logger.info("Creating Secops Role Manager")
            # TODO this is broken at the moment -> will fix in DSER-17408
            # Create Custom Connector
            # role_manager_connector_request = json.loads("{ \"name\":\"SECOPS_ROLE_MANAGER\", \"connectorType\":\"CUSTOM\", " \
            #                                  "\"authorizedIpAddresses\":[], \"description\":\"\", \"changedBy\":\"ops@carbonblack.com\" }")
            #
            # appservices_response = appservices_client.post(path='/appservices/org/1/settings/connector/add', payload=role_manager_connector_request)
            # appservices_response.raise_for_status()
            # manager_api_key = appservices_response.json()['entries'][0]['apiKey']
            # manager_connector_id = appservices_response.json()['entries'][0]['connectorId']
            #
            # auth_token = "%s/%s" % (manager_api_key, manager_connector_id)
            # logger.info("X-Auth-Token: %s/%s for role manager connector" % (manager_api_key, manager_connector_id))
            # org_key = get_org_key(dashboard_url, auth_token)
            #
            # # Associate Custom Connector with Grant of Secops Role Manager Role
            # role_manager_grant_definition = json.loads("{\"version\": {\"N\": \"1\"},\"principal\": {\"S\": \"PRINCIPAL\"}," \
            #                                 "\"org_ref\": {\"S\": \"ORGREF\"},\"roles\": " \
            #                                 "{\"SS\": [\"psc:role::SECOPS_ROLE_MANAGER\"]},\"revoked\": {\"N\": \"0\"}}")
            # role_manager_grant_definition['principal']['S'] = "psc:cnn:{}:{}".format(org_key, manager_connector_id)
            # role_manager_grant_definition['org_ref']['S'] =  "psc:org:{}".format(org_key)
            #
            # if green_only:
            #     logger.info("Creating Grant for Role Manager in %s environment" % env)
            #     dynamo_client.put_item("{}_access_grants".format(env), role_manager_grant_definition)
            #
            # dynamo_client.put_item("{}_access_grants".format(hub), role_manager_grant_definition)
            return connector_dict
    except requests.HTTPError as ex:
        logger.error("Request failed (json=%s)" % ex.response.__dict__, exc_info=ex)


def get_org_key(dashboard_url, auth_token):
    try:
        dashboard_client = app_clients.BaseClient(dashboard_url)
        dashboard_client.setData()
        dashboard_client.headers['X-Auth-Token'] = auth_token
        connector_login_response = dashboard_client.post(path="/appservices/v5/connector/login")
        connector_login_response.raise_for_status()

        connector_login_json = connector_login_response.json()
        org_id = connector_login_json['currentOrgId']
        if (org_id != 1) :
            raise ValueError('Connector not in org 1')
        org_key = connector_login_json['currentOrgKey']
        return org_key
    except requests.HTTPError as ex:
        logger.error("Request failed (json=%s)" % ex.response.__dict__, exc_info=ex)


def verify_correct_grant(dynamo_client, org_key, connector_id, hub):
    correct_role = 'psc:role::SECOPS_ROLE_MANAGER'
    try:
        manager_grant = dynamo_client.get_item("{}_access_grants".format(hub), {"principal" : {"S": "psc:cnn:{}:{}".format(org_key, connector_id)},
                                                                              "org_ref": {"S" :"psc:org:{}".format(org_key)}})
        current_role = manager_grant['Item']['roles']['SS'][0]
        if manager_grant['Item']['roles']['SS'][0] != correct_role:
            logger.info("Grant for Manager service connector not assigned correct role, assigned to role %s", current_role)
            manager_grant['Item']['roles']['SS'][0] = correct_role
            dynamo_client.put_item("{}_access_grants".format(hub), manager_grant['Item'])
            logger.info("Assigned correct psc:role::SECOPS_ROLE_MANAGER role to connector")
        else:
            logger.info("Manager service connector role is assigned correctly to %s", correct_role)
    except ClientError as ex:
        logger.error("Dynamo Request failed for role manager, error: %s)" % ex.response)

def get_hub(environment):
    hub = None
    localdev_hub = ['localdev']
    dev_hub = ['dev01']
    test_hub = ['test03']
    prod_us_hub = ['eap01', 'prod01', 'prod02', 'prod04', 'prod05']
    prod_eu_hub = ['prod06']
    prod_ap_hub = ['prodnrt']
    prod_ap2_hub = ['prodsyd']

    if environment in localdev_hub:
        hub = 'localdev'
    elif environment in dev_hub:
        hub = 'hub-dev-us'
    elif environment in test_hub:
        hub = 'hub04'
    elif environment in prod_us_hub:
        hub = 'hub-prod-us'
    elif environment in prod_eu_hub:
        hub = 'hub-prod-eu'
    elif environment in prod_ap_hub:
        hub = 'hub-prod-ap'
    elif environment in prod_ap2_hub:
        hub = 'hub-prod-ap2'

    if hub == None:
        raise Exception("Could not get Hub configuration for environment {}".format(environment))
    logger.info("Hub Environment %s " % hub)
    return hub

if __name__ == "__main__":
    main()